local config = require "config"
local helper = require "helper"
local UEHelpers = require "UEHelpers"

local polaris_hud = nil

if config.disable_everything then
    return
end

function hideItemsMainMenu(main_menu)

    --[[

    main_menu.Rep_T_UI_RNK_S:SetVisibility(2)

    if config.hide_battle_pass_info then
        local battle_pass = FindFirstOf("WBP_UI_BlackPen_MM_Banner_C")
        if battle_pass:IsValid() then
            battle_pass:SetVisibility(2)
            print("Battle Pass hidden")
        else
            print("Battle pass not found")
        end
    end

    ]]

end

function hideItemsRoundStart()

    local PolarisBlueprintLibrary = StaticFindObject("/Script/Polaris.Default__PolarisBlueprintLibrary")
    if not PolarisBlueprintLibrary:IsValid() then error("PolarisBlueprintLibrary not valid, Cannot patch the box!\n") end

    if  config.preferred_side ~= "LEFT" and config.preferred_side ~= "RIGHT" then
        error("Preferred side has not been set properly, changes will not be made!") end

    local _character_manager = PolarisBlueprintLibrary:GetCharacterManager()

    -- Code to loop between both characters.
    for i = 1, 2 do
        -- Character index used in-game.
        local _player_index = i - 1

        -- Locate Tekken Character Manager and retrieve the correct character
        local _character_actor = _character_manager:GetCharacterActor(_player_index)

        -- Run through configurations to hide things we dont want.
        if config.hide_player_auras then
            local found = hideAura(_character_actor)
            if found then
                print("Aura's have been hidden.")
            end
        end

        if config.hide_accessories then
            local found = hideAccessories(_character_actor)
            if found then
                print("Accessories have been hidden.")
            end
        end
    end

    polaris_hud = FindFirstOf("WBP_UI_HUD_C")
    if polaris_hud:IsValid() then
        adjustHudOpacity()

        if config.use_custom_title ~= "None" and config.use_custom_title ~= "" then
            print(string.format("Using Custom Title: %s", config.use_custom_title))
            useCustomTitle()
        else
            print("No Custom title was present so a title has not been added!")
        end

        if config.hide_special_style_menu then
            local control_guide = FindFirstOf("WBP_UI_HUD_ControlGuide_C")
            control_guide:SetRenderOpacity(0.0)

            print("Special Style options have been hidden!")
        end
    end
end

-- Multipurpose hide function that can be used to hide a lot of components.
function simpleHide(componentClass, CharacterActor)

    if not componentClass:IsValid() then error("There was an issue trying to hide a component class, check code again.\n") end
    local _actor_components = CharacterActor:K2_GetComponentsByClass(componentClass)

    local _item_hide = false

    for i, v in ipairs(_actor_components) do

        local _component = v:get()

        _component:SetHiddenInGame(true, false)
        _item_hide = true
    end

    return _item_hide
end

function hideAura(CharacterActor)

    local NiagaraComponent = StaticFindObject("/Script/Niagara.NiagaraComponent")
    local _actor_components = CharacterActor:K2_GetComponentsByClass(NiagaraComponent)

    local _aura_found = false

    for i, v in ipairs(_actor_components) do

        local _component = v:get()
        local _asset_name = _component:GetAsset():GetFullName()

        if not string.match(_asset_name, "Polariswork/Common/NiagaraSystem/") then

            -- print(string.format("Found and Replaced: %s", _asset_name))
            
            _component:SetHiddenInGame(true, false)
            _aura_found = true
        end
    end

    return _aura_found
end

function hideAccessories(CharacterActor)

    local GameplayStatics = UEHelpers:GetGameplayStatics()
    local MeshComponentStatic = StaticFindObject("/Script/Engine.MeshComponent")

    local _accessory_found = false

    -- Loop through and find any Static meshes.
    local _mesh_components = CharacterActor:K2_GetComponentsByClass(MeshComponentStatic)
    for i, v in ipairs(_mesh_components) do

        local _mesh = v:get()

        local _mesh_name = "None"
        local _mesh_class_name = GameplayStatics:GetObjectClass(_mesh):GetFullName()

        local _mesh_type = 0

        -- Check if it's a skeletal mesh or a static mesh.
        if _mesh_class_name == "Class /Script/Engine.SkeletalMeshComponent" then
            if _mesh.SkeletalMeshAsset:IsValid() then
                _mesh_name = _mesh.SkeletalMeshAsset:GetFullName()
                _mesh_type = 1
            end

        elseif _mesh_class_name == "Class /Script/Engine.StaticMeshComponent" then
            if _mesh.StaticMesh:IsValid() then
                _mesh_name = _mesh.StaticMesh:GetFullName()
                _mesh_type = 2
            end
        end

        if _mesh_type ~= 0 then
            local _accesory_check = "none"
            if config.hide_special_accessories then
                _accesory_check = "_acc_"
            else
                _accesory_check = "cmn_acc_"
            end
            -- If the mesh type is a type we should change, and if it's actually an accessory.
            if string.match(_mesh_name, _accesory_check) then

                print(string.format("Found and Replaced: %s", _mesh_name))

                _mesh:SetHiddenInGame(true, false)
                _accessory_found = true
            end
        end
    end

    return _accessory_found
end

function adjustHudOpacity()
    polaris_hud.CanvasPanel_277:SetRenderOpacity(config.hud_opacity)
end

function useCustomTitle()
    local player_hud = polaris_hud.ref_player

    if player_hud:IsValid() then
        local _shogo_panel = nil
        if config.preferred_side == "LEFT" then
            _shogo_panel = player_hud.WBP_UI_ShogoPanel_L
        else
            _shogo_panel = player_hud.WBP_UI_ShogoPanel_R
        end

        if _shogo_panel:IsValid() then
            _shogo_panel:SetShogo(FText(config.use_custom_title))

            print("Custom Title has been updated!")
        end
    end
end

function printCharacter()
    print("Getting character on your side")

    local PolarisBlueprintLibrary = StaticFindObject("/Script/Polaris.Default__PolarisBlueprintLibrary")
    if not PolarisBlueprintLibrary:IsValid() then error("PolarisBlueprintLibrary not valid, Cannot patch the box!\n") end

    if  config.preferred_side ~= "LEFT" and config.preferred_side ~= "RIGHT" then
        error("Preferred side has not been set properly, changes will not be made!") end

    local _character_manager = PolarisBlueprintLibrary:GetCharacterManager()
    local _character_actor = _character_manager:GetCharacterActor(0)

    print(string.sub(_character_actor:GetFullName(), 1, 16))
end



function printActorComponents()
    local PolarisBlueprintLibrary = StaticFindObject("/Script/Polaris.Default__PolarisBlueprintLibrary")
    if not PolarisBlueprintLibrary:IsValid() then error("PolarisBlueprintLibrary not valid, Cannot patch the box!\n") end

    local _character_manager = PolarisBlueprintLibrary:GetCharacterManager()
    local _character_actor = _character_manager:GetCharacterActor(0)

    local ActorComponent = StaticFindObject("/Script/Engine.ActorComponent")
    local NiagaraComponent = StaticFindObject("/Script/Niagara.NiagaraComponent")
    local PolarisTraceAttachComponent = StaticFindObject("/Script/Polaris.PolarisTraceAttachComponent")
    local PolarisTraceComponent = StaticFindObject("/Script/Polaris.PolarisTraceComponent")
    local PolarisMasterSkeletalMeshComponent = StaticFindObject("/Script/Polaris.PolarisMasterSkeletalMeshComponent")

    if not NiagaraComponent:IsValid() then error("Niagara Component is not valid!\n") end
    if not _character_actor:IsValid() then error("Character is not valid!\n") end

    local _actor_components = _character_actor:K2_GetComponentsByClass(NiagaraComponent)
    local _amount = 0

    for i, v in ipairs(_actor_components) do

        local _component = v:get()

        print(_component:GetAsset():GetFullName())
        print(string.format("Visibility: %s", _component.bVisible))
        print(string.format("Hidden In Game: %s", _component.bHiddenInGame))

        _amount = _amount + 1
    end

    print(string.format("Items Found: %s", _amount))
end

function applyDepthOfField()
    local PostProcessActors = FindAllOf("PolarisPostProcessVolume")

    local _custom_post_process = {
        ["DepthOfFieldFstop"] = 2.0,
        ["DepthOfFieldMinFstop"] = 11.0,
        ["DepthOfFieldFocalDistance"] = 60.0,
    }


    if not PostProcessActors then
        print("No instances of 'Actor' were found\n")
    else
        for i, v in ipairs(PostProcessActors) do
            -- v.BlendWeight = 1

            v.Settings.DepthOfFieldDepthBlurAmount = 90.0

            print(tostring(v.Settings.DepthOfFieldDepthBlurAmount))
        end
    end

    print("Applying depth of field")

end

RegisterHook("/Game/UI/Widget/HUD/WBP_UI_HUD_Player.WBP_UI_HUD_Player_C:SetZoneChainVisibility", function()
    hideItemsRoundStart()
end)

-- When your promotion is successful, the rank pop-up that happens.
NotifyOnNewObject("/Script/Polaris.PolarisUMGMainMenu", function(main_menu)
    print("Run hide code for main menu")
    ExecuteWithDelay(500, function()
        hideItemsMainMenu(main_menu)
    end)
end)

-- RegisterKeyBind(Key.F6, printActorComponents)
-- RegisterKeyBind(Key.F7, hideItemsRoundStart)